//	MIPS_ALU.v
/*
 *	A 4-bit ALU based on the MIPS ALU developed in
 *	Appendix B of Patterson & Hennessy, Computer Organization
 *  and Design.
 *
 *	C. Vickery, October 2008
 */
module MIPS_ALU(
	input  wire [3:0] A, B, Func,
	output wire [3:0] Result, CVNZ);

	wire [4:1] Carry;
	wire       isLess;
	
	Bit_Slice slice_0(.Ain(A[0]), .Bin(B[0]), .Cin(Func[2]),
		.Ainv(Func[3]), .Bneg(Func[2]), .F1(Func[1]), .F0(Func[0]),
		.SLTin(isLess), .Result(Result[0]), .Cout(Carry[1]) );

	Bit_Slice slice_1(.Ain(A[1]), .Bin(B[1]), .Cin(Carry[1]),
		.Ainv(Func[3]), .Bneg(Func[2]), .F1(Func[1]), .F0(Func[0]),
		.SLTin(1'b0), .Result(Result[1]), .Cout(Carry[2]) );

	Bit_Slice slice_2(.Ain(A[2]), .Bin(B[2]), .Cin(Carry[2]),
		.Ainv(Func[3]), .Bneg(Func[2]), .F1(Func[1]), .F0(Func[0]),
		.SLTin(1'b0), .Result(Result[2]), .Cout(Carry[3]) );

	Bit_Slice slice_3(.Ain(A[3]), .Bin(B[3]), .Cin(Carry[3]),
		.Ainv(Func[3]), .Bneg(Func[2]), .F1(Func[1]), .F0(Func[0]),
		.SLTin(1'b0), .SLTout(isLess),
		.Result(Result[3]), .Cout(Carry[4]) );

	assign CVNZ[3] = Carry[4];
	assign CVNZ[2] = Carry[4] ^ Carry[3];
	assign CVNZ[1] = Result[3];
	assign CVNZ[0] = !Result;

endmodule